@@ -39,9 +39,11 @@ static const int computedProperties[] = {
     CSS_PROP_BACKGROUND_IMAGE,
     CSS_PROP_BACKGROUND_REPEAT,
     CSS_PROP_BACKGROUND_ATTACHMENT,
+    CSS_PROP_BACKGROUND_ORIGIN,
     CSS_PROP_BACKGROUND_POSITION,
     CSS_PROP_BACKGROUND_POSITION_X,
     CSS_PROP_BACKGROUND_POSITION_Y,
+    CSS_PROP_BACKGROUND_SIZE,
     CSS_PROP_BORDER_COLLAPSE,
     CSS_PROP_BORDER_SPACING,
     CSS_PROP__KHTML_BORDER_HORIZONTAL_SPACING,
@@ -458,22 +460,24 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
             Q_ASSERT( 0 );
         }
         break;
+    case CSS_PROP_BACKGROUND_ORIGIN:
+        switch (style->backgroundLayers()->backgroundOrigin()) {
+            case BGBORDER:
+                return new CSSPrimitiveValueImpl(CSS_VAL_BORDER_BOX);
+            case BGPADDING:
+                return new CSSPrimitiveValueImpl(CSS_VAL_PADDING_BOX);
+            case BGCONTENT:
+                return new CSSPrimitiveValueImpl(CSS_VAL_CONTENT_BOX);
+        }
+        Q_ASSERT(0);
+        break;
     case CSS_PROP_BACKGROUND_POSITION:
     {
         RETURN_NULL_ON_NULL(renderer);
-        DOMString string;
-        Length length(style->backgroundXPosition());
-        if (length.isPercent())
-            string = QString(QString::number(length.percent()) + "%");
-        else
-            string = QString(QString::number(length.minWidth(renderer->contentWidth())) + "px");
-        string += " ";
-        length = style->backgroundYPosition();
-        if (length.isPercent())
-            string += QString(QString::number(length.percent()) + "%");
-        else
-            string += QString(QString::number(length.minWidth(renderer->contentWidth())) + "px");
-        return new CSSPrimitiveValueImpl(string, CSSPrimitiveValue::CSS_STRING);
+        CSSValueListImpl *values = new CSSValueListImpl(CSSValueListImpl::Space);
+        values->append(valueForLength(style->backgroundXPosition(), renderer->contentWidth()));
+        values->append(valueForLength(style->backgroundYPosition(), renderer->contentHeight()));
+        return values;
     }
     case CSS_PROP_BACKGROUND_POSITION_X:
         RETURN_NULL_ON_NULL(renderer);
@@ -481,6 +485,43 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
     case CSS_PROP_BACKGROUND_POSITION_Y:
         RETURN_NULL_ON_NULL(renderer);
         return valueForLength(style->backgroundYPosition(), renderer->contentHeight());
+    case CSS_PROP_BACKGROUND_SIZE: {
+        const EBackgroundSizeType backgroundSizeType = style->backgroundLayers()->backgroundSizeType();
+        switch (backgroundSizeType) {
+            case BGSCONTAIN:
+                return new CSSPrimitiveValueImpl(CSS_VAL_CONTAIN);
+            case BGSCOVER:
+                return new CSSPrimitiveValueImpl(CSS_VAL_COVER);
+            case BGSLENGTH: {
+                const LengthSize bgLengthSize = style->backgroundLayers()->backgroundSize();
+                CSSValueListImpl *values = new CSSValueListImpl(CSSValueListImpl::Space);
+                switch (bgLengthSize.width.type()) {
+                    case Auto:
+                        values->append(new CSSPrimitiveValueImpl(CSS_VAL_AUTO));
+                        break;
+                    case Percent:
+                        values->append(new CSSPrimitiveValueImpl(bgLengthSize.width.percent(), CSSPrimitiveValue::CSS_PERCENTAGE));
+                        break;
+                    default:
+                        values->append(new CSSPrimitiveValueImpl(bgLengthSize.width.value(), CSSPrimitiveValue::CSS_PX));
+                }
+                switch (bgLengthSize.height.type()) {
+                    case Auto:
+                        values->append(new CSSPrimitiveValueImpl(CSS_VAL_AUTO));
+                        break;
+                    case Percent:
+                        values->append(new CSSPrimitiveValueImpl(bgLengthSize.height.percent(), CSSPrimitiveValue::CSS_PERCENTAGE));
+                        break;
+                    default:
+                        values->append(new CSSPrimitiveValueImpl(bgLengthSize.height.value(), CSSPrimitiveValue::CSS_PX));
+                }
+                return values;
+            }
+            default:
+                Q_ASSERT(0);
+        }
+        break;
+    }
     case CSS_PROP_BORDER_COLLAPSE:
         if (style->borderCollapse())
             return new CSSPrimitiveValueImpl(CSS_VAL_COLLAPSE);
@@ -488,18 +529,15 @@ CSSValueImpl *RenderStyleDeclarationImpl::getPropertyCSSValue( int propertyID )
             return new CSSPrimitiveValueImpl(CSS_VAL_SEPARATE);
     case CSS_PROP_BORDER_SPACING:
     {
-        QString string(QString::number(style->borderHorizontalSpacing()) +
-                       "px " +
-                       QString::number(style->borderVerticalSpacing()) +
-                       "px");
-        return new CSSPrimitiveValueImpl(DOMString(string), CSSPrimitiveValue::CSS_STRING);
+        CSSValueListImpl *values = new CSSValueListImpl(CSSValueListImpl::Space);
+        values->append(new CSSPrimitiveValueImpl(style->borderHorizontalSpacing(), CSSPrimitiveValue::CSS_PX));
+        values->append(new CSSPrimitiveValueImpl(style->borderVerticalSpacing(), CSSPrimitiveValue::CSS_PX));
+        return values;
     }
     case CSS_PROP__KHTML_BORDER_HORIZONTAL_SPACING:
-        return new CSSPrimitiveValueImpl(style->borderHorizontalSpacing(),
-                                         CSSPrimitiveValue::CSS_PX);
+        return new CSSPrimitiveValueImpl(style->borderHorizontalSpacing(), CSSPrimitiveValue::CSS_PX);
     case CSS_PROP__KHTML_BORDER_VERTICAL_SPACING:
-        return new CSSPrimitiveValueImpl(style->borderVerticalSpacing(),
-                                         CSSPrimitiveValue::CSS_PX);
+        return new CSSPrimitiveValueImpl(style->borderVerticalSpacing(), CSSPrimitiveValue::CSS_PX);
     case CSS_PROP__KHTML_BORDER_TOP_RIGHT_RADIUS:
     case CSS_PROP_BORDER_TOP_RIGHT_RADIUS:
 	return valueForBorderRadii(style->borderTopRightRadius());
