@@ -55,7 +55,7 @@ void HTMLBaseElementImpl::parseAttribute(AttributeImpl *attr)
     switch(attr->id())
     {
     case ATTR_HREF:
-	m_href = attr->value().string().trimmed();
+	m_href = attr->value().trimSpaces().string();
 	process();
 	break;
     case ATTR_TARGET:
@@ -116,9 +116,9 @@ void HTMLLinkElementImpl::parseAttribute(AttributeImpl *attr)
     switch (attr->id())
     {
     case ATTR_HREF: {
-        const QString hrefUrl = attr->val()->string().trimmed();
+        const DOMString hrefUrl = attr->value().trimSpaces();
         if (!hrefUrl.isEmpty()) {
-            m_url = document()->completeURL(hrefUrl);
+            m_url = document()->completeURL(hrefUrl.string());
         }
         process();
         break;
@@ -385,7 +385,7 @@ void HTMLScriptElementImpl::parseAttribute(AttributeImpl *attr)
         // we'll only start doing things once we get insertedIntoDocument()
         if (m_evaluated || m_cachedScript || m_createdByParser || !inDocument())
             return;
-        QString url = attr->value().string().trimmed();
+        const DOMString url = attr->value().trimSpaces();
         if (!url.isEmpty())
             loadFromUrl(url);
         break;
@@ -490,7 +490,7 @@ void HTMLScriptElementImpl::insertedIntoDocument()
     if (m_createdByParser)
         return;
 
-    QString url = getAttribute(ATTR_SRC).string().trimmed();
+    const DOMString url = getAttribute(ATTR_SRC).trimSpaces();
     if (!url.isEmpty()) {
         loadFromUrl(url);
         return;
@@ -622,7 +622,7 @@ void HTMLScriptElementImpl::setDefer(bool defer)
 
 DOMString HTMLScriptElementImpl::src() const
 {
-    return document()->completeURL(getAttribute(ATTR_SRC).string().trimmed());
+    return document()->completeURL(getAttribute(ATTR_SRC).trimSpaces().string());
 }
 
 void HTMLScriptElementImpl::setSrc(const DOMString &value)
